//--------------------------------------------------------------------------//
// Title:       golden_top.v                                                //
//--------------------------------------------------------------------------//
// Description: Golden Top file contains Cyclone III F780 Host Board pins   //
//              and I/O Standards for 3C120F780 device.  Matches rev B and  //
//              rev C circuit board.  Note certain pins that are intended   //
//              to be driven open-drain or have a TOGGLE_RATE=0 to have a   //
//              successful fit in Quartus.  20071031                        //
//--------------------------------------------------------------------------//
//------ 1 ------- 2 ------- 3 ------- 4 ------- 5 ------- 6 ------- 7 ------7
//------ 0 ------- 0 ------- 0 ------- 0 ------- 0 ------- 0 ------- 0 ------8
//----------------------------------------------------------------------------
//Copyright © 2007 Altera Corporation. All rights reserved.  Altera products
//are protected under numerous U.S. and foreign patents, maskwork rights,
//copyrights and other intellectual property laws.
//                 
//This reference design file, and your use thereof, is subject to and
//governed by the terms and conditions of the applicable Altera Reference
//Design License Agreement.  By using this reference design file, you
//indicate your acceptance of such terms and conditions between you and
//Altera Corporation.  In the event that you do not agree with such terms and
//conditions, you may not use the reference design file. Please promptly 
//destroy any copies you have made.
//
//This reference design file being provided on an "as-is" basis and as an
//accommodation and therefore all warranties, representations or guarantees
//of any kind (whether express, implied or statutory) including, without
//limitation, warranties of merchantability, non-infringement, or fitness for
//a particular purpose, are specifically disclaimed.  By making this
//reference design file available, Altera expressly does not recommend,
//suggest or require that this reference design file be used in combination 
//with any other product not provided by Altera


`define DDR2_TOP
//`define DDR2_BOT
`define DDR2_64BIT_ONLY
`define FSM
//`define HSMC_A
//`define HSMC_A_LVDS
`define HSMC_A_CMOS
`define HSMC_B
//`define HSMC_B_LVDS
`define HSMC_B_CMOS
//`define USER_IO_SEVEN_SEG
`define USER_IO_LED
`define USER_IO_DIPSW
`define USER_IO_PB
//`define LCD
//`define USB
`define ENET


module niosII_cycloneIII_3c120_host_board_top (
//CLK-Signals--------------------------//////4 pins
    input          clkin_50,           //1.8V   //clk input pin
    input          clkin_125,          //1.8V   //clk input pin
    input          clkin_sma,          //1.8V   //clk input pin
    output         clkout_sma,         //1.8V   //clk output pin

//Dual-Channel-DDR2--------------------//////148 pins
`ifdef DDR2_TOP
    //Top-edge address/control pins for DQ(39:0)
    output [15:0]  ddr2top_a,          //1.8V
    output [2:0]   ddr2top_ba,         //1.8V
    output         ddr2top_casn,       //1.8V
    output         ddr2top_cke,        //1.8V
    output         ddr2top_csn,        //1.8V
    output         ddr2top_odt,        //1.8V
    output         ddr2top_rasn,       //1.8V
    output         ddr2top_wen,        //1.8V
    output         ddr2top_active,     //1.8V   //open-drain LED
`endif
`ifdef DDR2_BOT
    //Bottom-edge address/control pins for DQ(71:40)
    output [15:0]  ddr2bot_a,          //1.8V
    output [2:0]   ddr2bot_ba,         //1.8V
    output         ddr2bot_casn,       //1.8V
    output         ddr2bot_cke,        //1.8V
    output         ddr2bot_csn,        //1.8V
    output         ddr2bot_odt,        //1.8V
    output         ddr2bot_rasn,       //1.8V
    output         ddr2bot_wen,        //1.8V
    output         ddr2bot_active,     //1.8V   //open-drain LED
`endif
    //DQ(39:0) uses DQS(4:0), DQ(71:40) uses DQS(8:5)
`ifdef DDR2_64BIT_ONLY
//    inout  [63:0]  ddr2_dq,            //1.8V
    inout  [63:32]  ddr2_dq,            //1.8V
    inout  [7:4]   ddr2_dqs,           //1.8V
    output [7:4]   ddr2_dm,            //1.8V
`else
    inout  [71:0]  ddr2_dq,            //1.8V
    inout  [8:0]   ddr2_dqs,           //1.8V
    output [8:0]   ddr2_dm,            //1.8V
`endif
    inout  [2:1]   ddr2_ck_p,          //1.8V
    inout  [2:1]   ddr2_ck_n,          //1.8V

//FLASH-SRAM-MAX-------------FSM-Bus---//////78 pins
`ifdef FSM
    output [24:0]  fsa,                //1.8V   //FSM Address Bus 
    inout  [31:0]  fsd,                //1.8V   //FSM Data Bus 
    output [3:0]   sram_ben,           //1.8V
    output         sram_advn,          //1.8V
    output         sram_clk,           //1.8V
    output         sram_csn,           //1.8V
    output         sram_oen,           //1.8V
    output         sram_psn,           //1.8V
    input  [1:0]   sram_wait,          //1.8V
    output         sram_wen,           //1.8V
    output         flash_cen,          //1.8V
    output         flash_oen,          //1.8V
    input          flash_rdybsyn,      //1.8V
    output         flash_resetn,       //1.8V
    output         flash_wen,          //1.8V
    output         max2_clk,           //1.8V
    output         max_csn,            //1.8V
    output         max_oen,            //1.8V
    output         max_wen,            //1.8V
`endif

//Ethernet-----------------------------//////16 pins
`ifdef ENET
    output         enet_gtx_clk,       //2.5V   //1000Mb TX Clock
    output         enet_tx_en,         //2.5V
    output [3:0]   enet_txd,           //2.5V
    input          enet_rx_clk,        //2.5V   //clk input pin
    input          enet_rx_dv,         //2.5V
    input  [3:0]   enet_rxd,           //2.5V
    output         enet_resetn,        //2.5V
    output         enet_mdc,           //2.5V
    inout          enet_mdio,          //2.5V   //open-drain
    input	   enet_led_link1000,  //2.5V
`endif

//LCD-Displays-------------------------//////14 pins
`ifdef LCD
    output         lcd_en,             //2.5V   //Char LCD select
    output         lcd_csn,            //2.5V   //Graphics LCD select
    output         lcd_rstn,           //2.5V
    output         lcd_d_cn,           //2.5V
    output         lcd_wen,            //2.5V
    output         lcd_e_rdn,          //2.5V
    inout  [7:0]   lcd_data,           //2.5V
`endif

//Speaker-Header-----------------------//////1 pin
    output         speaker_out,        //2.5V
//USB-Data-Port------------------------//////14 pins
    //synchronous master FIFO handshake mode
`ifdef USB
    inout  [7:0]   usb_fd,             //2.5V
    input          usb_ifclk,          //1.8V   //clk input pin
    input          usb_cmd_data,       //2.5V
    input          usb_ren,            //2.5V
    input          usb_wen,            //2.5V
    output         usb_empty,          //2.5V
    output         usb_full,           //2.5V
`endif

//HSMC-Channel-A-----------------------//////86 pins
  //Enable busses below for LVDS HSMC
`ifdef HSMC_A
`ifdef HSMC_A_LVDS
    output [16:0]  hsma_tx_d_p,        //LVDS
    input  [16:0]  hsma_rx_d_p,        //LVDS
    inout  [3:0]   hsma_d,             //2.5V
    input  [2:1]   hsma_clk_in_p,      //LVDS
    output [2:1]   hsma_clk_out_p,     //LVDS
`endif
  //Enable below for CMOS HSMC
`ifdef HSMC_A_CMOS
  inout	[35:0]	hsma_d_a,             //2.5V
  inout	[75:40]	hsma_d_b,             //2.5V
  inout	[2:1]	hsma_clk_out_p_cmos,
  inout	[2:1]	hsma_clk_out_n_cmos,
  input	[2:1]	hsma_clk_in_p_cmos,
  input	[2:1]	hsma_clk_in_n_cmos,           
`endif
  //Dedicated clocks and controls
    input          hsma_clk_in0,       //1.8V   //clk input pin
    output         hsma_clk_out0,      //2.5V
    inout          hsma_sda,           //2.5V   //open-drain
    output         hsma_scl,           //2.5V
    output         hsma_tx_led,        //2.5V
    output         hsma_rx_led,        //2.5V
`endif
//HSMC-Channel-B-----------------------//////86 pins
  //Enable below for LVDS HSMC
`ifdef HSMC_B
`ifdef HSMC_B_LVDS
    output [16:0]  hsmb_tx_d_p,        //LVDS
    input  [16:0]  hsmb_rx_d_p,        //LVDS
    inout  [3:0]   hsmb_d,             //2.5V
    input  [2:1]   hsmb_clk_in_p,      //LVDS
    output [2:1]   hsmb_clk_out_p,     //LVDS
`endif
  //Enable below for CMOS HSMC
`ifdef HSMC_B_CMOS                           
  inout	[35:0]	hsmb_d_a,             //2.5V
  inout	[75:40]	hsmb_d_b,             //2.5V
  inout	[2:1]	hsmb_clk_out_p_cmos,
  inout	[2:1]	hsmb_clk_out_n_cmos,
  input	[2:1]	hsmb_clk_in_p_cmos,
  input	[2:1]	hsmb_clk_in_n_cmos,           
`endif
  //Dedicated clocks and controls
    input          hsmb_clk_in0,       //1.8V   //clk input pin
    output         hsmb_clk_out0,      //2.5V
    inout          hsmb_sda,           //2.5V   //open-drain
    output         hsmb_scl,           //2.5V
    output         hsmb_tx_led,        //2.5V
    output         hsmb_rx_led,        //2.5V
`endif

//User-IO------------------------------//////34 pins
`ifdef USER_IO_SEVEN_SEG
    output         seven_seg_a,        //1.8V   //open-drain
    output         seven_seg_b,        //1.8V   //open-drain
    output         seven_seg_c,        //1.8V   //open-drain
    output         seven_seg_d,        //1.8V   //open-drain
    output         seven_seg_e,        //1.8V   //open-drain
    output         seven_seg_f,        //1.8V   //open-drain
    output         seven_seg_g,        //1.8V   //open-drain
    output         seven_seg_dp,       //1.8V   //open-drain
    output         seven_seg_minus,    //1.8V   //open-drain
    output [4:1]   seven_seg_sel,      //1.8V   //open-drain
`endif
`ifdef USER_IO_DIPSW
    input  [7:0]   user_dipsw,         //1.8V
`endif
`ifdef USER_IO_LED
    output [7:0]   user_led,           //1.8V   //open-drain
`endif
`ifdef USER_IO_PB
    input  [3:0]   user_pb,            //1.8V
`endif
    input          cpu_resetn         //2.5V   //DEV_CLR pin

);
/*****************************************************************************/
//	Stuff for Neek LCD interface on HSMC Port B
/*****************************************************************************/
	//wires for NEEK LCD HSMC
	//Eth Interface
	wire				local_eth_reset_n;
	wire				local_eth_tx_clk;
	wire				local_eth_tx_en;                            
	wire	[3:0]			local_eth_tx_d;
	wire				local_eth_rx_clk;
	wire				local_eth_rx_dv;
	wire				local_eth_rx_crs;
	wire				local_eth_rx_err;
	wire				local_eth_rx_col;
	wire	[3:0]			local_eth_rx_d;
	wire				local_eth_mdio_in;
	wire				local_eth_mdio_out;
	wire				local_eth_mdio_oen;
	wire				local_eth_mdc;
	
	//VGA Interface
	wire	[9:0]			local_vga_data; 
	wire				local_vga_blank;        
	wire				local_vga_sync;
	wire				local_vga_hs;  
	wire				local_vga_vs;  
	wire				local_vga_clk;
	
	//PS2 Interface   
	wire				local_ps2_clk;
	wire				local_ps2_dat;
	
	//Audio Interface                                                      
	wire				local_aud_bclk;
	wire				local_aud_xck;
	wire				local_aud_dacdat;               
	wire				local_aud_daclrck;
	wire				local_aud_adcdat;
	wire				local_aud_adclrck;
	
	//LCD Interface                                                            
	wire				local_lcd_vd;
	wire				local_lcd_den;
	wire				local_lcd_hd;
	wire				local_lcd_nclk;
	wire	[7:0]			local_lcd_data;
	wire				local_lcd_reset_n;
	wire				local_lcd_scen;  
	wire				local_lcd_sda;
	wire				local_lcd_scl;
	
	//SDMMC Interface 
	wire				local_sd_cmd;
	wire				local_sd_dat3;
	wire				local_sd_dat;
	wire				local_sd_clk;
	
	//UART Interface                                           
	wire				local_uart_rxd;
	wire				local_uart_txd;
	
	//Vid Dec Interface
	wire	[7:0]			local_vid_dec_td_d;
	wire				local_vid_dec_td_27mhz;
	wire				local_vid_dec_td_reset;
	wire				local_vid_dec_td_vs;
	wire				local_vid_dec_td_hs; 
	
	//Touch Pad       
	wire				local_tp_adc_dclk;
	wire				local_tp_adc_cs_n;
	wire				local_tp_adc_din;
	wire				local_tp_adc_dout;
	wire				local_tp_adc_penirq_n;
	wire				local_tp_adc_busy;
	
	//I2C for a bunch of components
	wire				local_i2c_sclk;
	wire				local_i2c_sdat;
	
	//I2C for ID
	wire				local_id_i2c_scl;
	wire				local_id_i2c_sdat; 	
	
	//Instantiate module for NEEK LCD HSMC
	hsmc_neek_lcd_interface hsmc_neek_lcd_interface_inst
	(
		//HSMC interface  
		.hsmc_clk_in0		(hsmb_clk_in0),
		.hsmc_clk_out0		(hsmb_clk_out0),
		.hsmc_sda		(hsmb_sda),                   
		.hsmc_scl		(hsmb_scl),          
		.hsmc_d_a		(hsmb_d_a),
		.hsmc_d_b		(hsmb_d_b),   
		.hsmc_clkout_p		(hsmb_clk_out_p_cmos),
		.hsmc_clkout_n		(hsmb_clk_out_n_cmos),
		.hsmc_clkin_p		(hsmb_clk_in_p_cmos),
		.hsmc_clkin_n		(hsmb_clk_in_n_cmos),
										      
		//Eth Interface
		.local_eth_reset_n	(1'b0),
		.local_eth_tx_clk	(),
		.local_eth_tx_en	(1'b0),                  
		.local_eth_tx_d		(4'bzzzz),
		.local_eth_rx_clk	(),
		.local_eth_rx_dv	(),
		.local_eth_rx_crs	(),
		.local_eth_rx_err	(),
		.local_eth_rx_col	(),
		.local_eth_rx_d		(),
		.local_eth_mdio_oen	(1'b1),
		.local_eth_mdio_in	(),
		.local_eth_mdio_out	(1'b1),
		.local_eth_mdc		(1'b0),
	
		//VGA Interface
		.local_vga_data		(10'bzzzzzzzzzz), 
		.local_vga_blank	(1'bz),        
		.local_vga_sync		(1'bz),
		.local_vga_hs		(1'bz),  
		.local_vga_vs		(1'bz),  
		.local_vga_clk		(1'bz),
		
		//PS2 Interface   
		.local_ps2_clk		(1'bz),
		.local_ps2_dat		(),
	
		//Audio Interface                                                      
		.local_aud_bclk		(1'bz),
		.local_aud_xck		(1'bz),
		.local_aud_dacdat		(1'bz),               
		.local_aud_daclrck		(1'bz),
		.local_aud_adcdat		(),
		.local_aud_adclrck		(1'bz),
		
		//LCD Interface   
		.local_lcd_vd		(1'bz),
		.local_lcd_den		(1'b0),
		.local_lcd_hd		(1'bz),
		.local_lcd_nclk		(1'bz),
		.local_lcd_data		(1'bz),
		.local_lcd_reset_n	(1'b0),
		.local_lcd_scen		(1'b1),
		.local_lcd_sda		(1'bz),
		.local_lcd_scl		(1'b0),
		
		//SDMMC Interface 
		.local_sd_cmd		(1'bz),
		.local_sd_dat3		(1'bz),
		.local_sd_dat		(),
		.local_sd_clk		(1'bz),
	
		//UART Interface                                           
		.local_uart_rxd		(local_uart_rxd),
		.local_uart_txd		(local_uart_txd),
		
		//Vid Dec Interface
		.local_vid_dec_td_d	(),
		.local_vid_dec_td_27mhz	(),
		.local_vid_dec_td_reset	(1'b1),
		.local_vid_dec_td_vs	(),
		.local_vid_dec_td_hs	(), 
		
		//Touch Pad       
		.local_tp_adc_dclk	(1'bz),
		.local_tp_adc_cs_n	(1'b1),
		.local_tp_adc_din	(1'bz),
		.local_tp_adc_dout	(),
		.local_tp_adc_penirq_n	(),
		.local_tp_adc_busy	(),
		
		//I2C for a bunch of components
		.local_i2c_sclk		(1'bz),                       
		.local_i2c_sdat		(1'bz),
		
		//I2C for ID
		.local_id_i2c_scl	(1'bz),
		.local_id_i2c_sdat	(1'bz)	 
	);
/*****************************************************************************/  
//	User Stuff
/*****************************************************************************/

	//ENET clocking
	wire rx_clock;
	wire gtx_clock;
	wire enet_ena_10;
	wire enet_eth_mode;

	tse_clocks tse_clocks_inst (
		.eth_mode	(enet_eth_mode),
		.ena_10		(enet_ena_10),
		.clkin_125mhz	(clkin_125),
		.rx_inclk	(enet_rx_clk),
		.rx_outclk	(rx_clock),
		.tx_clk_core	(gtx_clock),
		.tx_clk_phy	(enet_gtx_clk)
	);
	
	//ENET stuff
	wire	enet_mdio_out;
	wire	enet_mdio_oen;
	assign	enet_mdio = enet_mdio_oen?1'bz:enet_mdio_out;
	//assign	enet_resetn = cpu_resetn;                   
	
	//flash switcharoo
	wire	[25:0]	fsa_w;
	assign	fsa[24:0] = fsa_w[25:1];
	
	//da nios
	niosII_cycloneIII_3c120_host_board_sopc niosII_cycloneIII_3c120_host_board_sopc_inst
	(
		//Globals      
		.reset_n				(cpu_resetn),  
		.clk					(clkin_50), 
//		.cpu_clk				(clk_100MHz),
//		.peripheral_clk				(clk_75MHz),
		
		//ddr top    
		.local_init_done_from_the_ddr2_top	(),
		.local_refresh_ack_from_the_ddr2_top	(),
		.local_wdata_req_from_the_ddr2_top	(),
		
		.global_reset_n_to_the_ddr2_top		(cpu_resetn),
	
		.reset_phy_clk_n_from_the_ddr2_top	(enet_resetn),	
		.mem_addr_from_the_ddr2_top		(ddr2top_a[15:0]),
		.mem_ba_from_the_ddr2_top		(ddr2top_ba[2:0]),
		.mem_cas_n_from_the_ddr2_top		(ddr2top_casn),
		.mem_cke_from_the_ddr2_top		(ddr2top_cke),
		.mem_clk_n_to_and_from_the_ddr2_top	(ddr2_ck_n[2:1]),
		.mem_clk_to_and_from_the_ddr2_top	(ddr2_ck_p[2:1]),
		.mem_cs_n_from_the_ddr2_top		(ddr2top_csn),
		.mem_dm_from_the_ddr2_top		(ddr2_dm[7:4]),    
		.mem_dq_to_and_from_the_ddr2_top	(ddr2_dq[63:32]),
		.mem_dqs_to_and_from_the_ddr2_top	(ddr2_dqs[7:4]),
		.mem_odt_from_the_ddr2_top		(ddr2top_odt),
		.mem_ras_n_from_the_ddr2_top		(ddr2top_rasn),
		.mem_we_n_from_the_ddr2_top		(ddr2top_wen),

		//TSE on board
	
		.rx_clk_to_the_tse_mac			(rx_clock),
		.tx_clk_to_the_tse_mac			(gtx_clock),
		.rgmii_in_to_the_tse_mac		(enet_rxd),
		.rgmii_out_from_the_tse_mac		(enet_txd),
		.rx_control_to_the_tse_mac		(enet_rx_dv),
		.set_1000_to_the_tse_mac		(),
		.set_10_to_the_tse_mac			(),
		.tx_control_from_the_tse_mac		(enet_tx_en),
		.ena_10_from_the_tse_mac		(enet_ena_10),
		.eth_mode_from_the_tse_mac		(enet_eth_mode),
		.mdc_from_the_tse_mac			(enet_mdc),
		.mdio_in_to_the_tse_mac			(enet_mdio),
		.mdio_oen_from_the_tse_mac		(enet_mdio_oen),
		.mdio_out_from_the_tse_mac		(enet_mdio_out),


		//PIOs   
		.out_port_from_the_led_pio		(user_led),
		//.out_port_from_the_seven_seg_pio	(sev_seg_bus),
		.in_port_to_the_button_pio		(user_pb),
		.in_port_to_the_dipsw_pio		(user_dipsw),   
		
		//Flash Bridge
		.fsm_tristate_bridge_address		(fsa_w),
		.fsm_tristate_bridge_data		(fsd[15:0]),
		.read_n_to_the_ext_flash		(flash_oen),
		.write_n_to_the_ext_flash		(flash_wen),
		.select_n_to_the_ext_flash		(flash_cen),
		
		//UART  
		.rxd_to_the_uart			(local_uart_rxd),
		.txd_from_the_uart			(local_uart_txd)		
	);
	
	//Flash
	assign	flash_resetn	= cpu_resetn;      
	assign	fsd[31:16]	= 16'bZZZZZZZZZZZZZZZZ;                
	//disable PSRAM
	assign	sram_ben	= 4'b1111;
	assign	sram_advn	= 1'b1;
	assign	sram_clk	= 1'b0;
	assign	sram_csn	= 1'b1;
	assign	sram_oen	= 1'b1;
	assign	sram_psn	= 1'b1;
	assign	sram_wen	= 1'b1;
	
	//disable MAX
	assign	max2_clk	= 1'b0;
	assign	max_csn		= 1'b1;
	assign	max_oen		= 1'b1;
	assign	max_wen		= 1'b1;    
	
	//LCD
	//Disable graphics LCD
	//assign	lcd_csn	=	1'b1;
	
endmodule
